﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

// Extra Namespaces we have imported
using System.Net;
using System.Net.Mail;
using System.Net.Configuration;
using System.Text.RegularExpressions;
using System.Configuration;
using System.Web.Configuration;
using System.Xml;
using System.Text;
using System.IO;

using umbraco;
using umbraco.presentation.nodeFactory;
using WebFormsMvp.Web;
using CodeGarden10.Logic.Views;
using CodeGarden10.Logic.Views.Models;
using WebFormsMvp;
using CodeGarden10.Logic.Presenters;

namespace umbracowebsitewizard_site.Usercontrols
{
    [PresenterBinding(typeof(ContactFormPresenter))]
    public partial class ContactForm : MvpUserControl<PageModel>, IContactFormView
    {

        #region Properties from Umbraco

        public string EmailTo { get; set; }

        public string EmailSubject { get; set; }

        public string EmailBody { get; set; }

        public string EmailReplyFrom { get; set; }

        public string EmailReplySubject { get; set; }

        public string EmailReplyBody { get; set; }

        public string FormHeader { get; set; }

        public string FormText { get; set; }

        public string ThankYouHeaderText { get; set; }

        public string ThankYouMessageText { get; set; }

        public bool EnableSSL { get; set; }

        #endregion

        #region Validation
        protected void RequiredName_ServerValidate(object source, ServerValidateEventArgs args)
        {
            if (name.Text == String.Empty)
            {
                name.CssClass = "error";
                args.IsValid = false;
            }
            else
            {
                name.CssClass = "";
                args.IsValid = true;
            }
        }


        protected void RequiredEmailAddress_ServerValidate(object source, ServerValidateEventArgs args)
        {
            if (emailAddress.Text == String.Empty)
            {
                emailAddress.CssClass = "error";
                args.IsValid = false;
            }
            else
            {
                emailAddress.CssClass = "";
                args.IsValid = true;
            }
        }

        protected void RequiredMessage_ServerValidate(object source, ServerValidateEventArgs args)
        {
            if (message.Text == String.Empty)
            {
                message.CssClass = "error";
                args.IsValid = false;
            }
            else
            {
                message.CssClass = "";
                args.IsValid = true;
            }
        }

        protected void CheckEmailAddress_ServerValidate(object source, ServerValidateEventArgs args)
        {

            //Email Regex String
            Regex reg = new Regex(@"\w+([-+.']\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*");

            // Check if there is NO match in Regex
            if (!reg.IsMatch(emailAddress.Text))
            {
                //Does not match RegEx - Invalid email address

                emailAddress.CssClass = "error";
                args.IsValid = false;
            }
            else
            {
                emailAddress.CssClass = "";
                args.IsValid = true;
            }
        }

        #endregion

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            this.btnSubmit.ServerClick += new EventHandler(btnSubmit_ServerClick);
        }

        void btnSubmit_ServerClick(object sender, EventArgs e)
        {
            var handler = this.Submit;
            if (handler != null)
            {
                handler(sender, new ContactUsEventArgs
                {
                    Address1 = this.addressLine1.Text,
                    Address2 = this.addressLine2.Text,
                    Email = this.emailAddress.Text,
                    Enquiry = this.message.Text,
                    Name = this.name.Text
                });
            }
        }

        #region IContactFormView Members


        public void EmailSendingFailure(string p)
        {
            throw new NotImplementedException();
        }

        public void EmailSent()
        {
            FormFields.Visible = false;         //Hide the FormFields DIV
            ErrorMailSettings.Visible = false;  //Hide the errormessage DIV
            ThankYou.Visible = true;            //SHOW our thankyou message DIV
        }

        #endregion

        #region ISubmittable<ContactUsEventArgs> Members

        public event EventHandler<ContactUsEventArgs> Submit;

        #endregion

        #region IValidatable Members

        public bool IsValid
        {
            get { return Page.IsValid; }
        }

        #endregion
    }
}

