﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Examine;
using UmbracoExamine.SearchCriteria;

namespace CodeGarden10.Web.usercontrols.website
{
    public partial class SearchResults : System.Web.UI.UserControl
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            string searchTerm = Request.QueryString["search"];
            if (!string.IsNullOrEmpty(searchTerm))
            {
                var searcher = ExamineManager.Instance.SearchProviderCollection["CWSSearcher"];

                var searchCriteria = searcher.CreateSearchCriteria();

                searchCriteria = searchCriteria
                    .GroupedOr((FieldsToSearchOn ?? "nodeName,headerText,nodeTypeAlias,bodyText" ).Split(','), searchTerm)
                    .Compile();

                ISearchResults results = searcher.Search(searchCriteria);

                this.TotalItemCount.Text = results.TotalItemCount.ToString();

                //Setup paging. If there isn't a page specified default to page 0
                int page = 0;
                int.TryParse(Request.QueryString["page"], out page);

                this.rptSearchResults.DataSource = results.Skip(page * ItemsPerPage).Take(ItemsPerPage == 0 ? 5 : ItemsPerPage);
                this.rptSearchResults.DataBind();
            }
        }

        protected string GenerateBlurb(SearchResult item)
        {
            var text = string.Empty;
            if (item.Fields.ContainsKey("bodyText"))
                text = item["bodyText"];
            else if (item.Fields.ContainsKey("headerText"))
                text = item["headerText"];
            else
                text = item["nodeName"];

            if (text.Length > 200)
                text = text.Substring(0, 197) + "...";

            return text;
        }

        public string FieldsToSearchOn { get; set; }
        public int ItemsPerPage { get; set; }
    }
}